@@ -1,22 +1,25 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "cc/paint/element_id.h"
 
 #include <inttypes.h>
+
 #include <limits>
 #include <ostream>
+#include <string>
 
 #include "base/strings/stringprintf.h"
 #include "base/trace_event/traced_value.h"
-#include "base/values.h"
 
 namespace cc {
 
-const ElementIdType ElementId::kInvalidElementId = 0;
-
 ElementId LayerIdToElementIdForTesting(int layer_id) {
+  // This intentionally returns an ElementId that is different from layer_id
+  // and is unlikely to conflict with other ElementIds. This ensures testing of
+  // our code not to depend on that the UI compositor uses ElementId(layer_id)
+  // as the element id of a layer.
   return ElementId(std::numeric_limits<int>::max() - layer_id);
 }
 
@@ -26,22 +29,12 @@ void ElementId::AddToTracedValue(base::trace_event::TracedValue* res) const {
   res->EndDictionary();
 }
 
-ElementIdType ElementId::GetInternalValue() const {
-  return id_;
-}
-
 std::string ElementId::ToString() const {
   return base::StringPrintf("(%" PRIu64 ")", id_);
 }
 
-std::unique_ptr<base::Value> ElementId::AsValue() const {
-  std::unique_ptr<base::DictionaryValue> res(new base::DictionaryValue());
-  res->SetInteger("id_", id_);
-  return std::move(res);
-}
-
 size_t ElementIdHash::operator()(ElementId key) const {
-  return std::hash<int>()(key.id_);
+  return std::hash<ElementId::InternalValue>()(key.id_);
 }
 
 std::ostream& operator<<(std::ostream& out, const ElementId& id) {
