@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -29,11 +29,9 @@ bool NativeMessagingHostListPolicyHandler::CheckListEntry(
 }
 
 void NativeMessagingHostListPolicyHandler::ApplyList(
-    std::unique_ptr<base::ListValue> filtered_list,
+    base::Value::List filtered_list,
     PrefValueMap* prefs) {
-  DCHECK(filtered_list);
-  prefs->SetValue(pref_path_,
-                  base::Value::FromUniquePtrValue(std::move(filtered_list)));
+  prefs->SetValue(pref_path_, base::Value(std::move(filtered_list)));
 }
 
 }  // namespace extensions
