@@ -1,58 +1,67 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CHROME_BROWSER_UI_WEBUI_PRINT_PREVIEW_PRINT_PREVIEW_HANDLER_H_
 #define CHROME_BROWSER_UI_WEBUI_PRINT_PREVIEW_PRINT_PREVIEW_HANDLER_H_
 
-#include <map>
 #include <memory>
-#include <set>
+#include <optional>
 #include <string>
+#include <vector>
 
+#include "base/containers/flat_map.h"
+#include "base/containers/flat_set.h"
 #include "base/files/file_path.h"
 #include "base/gtest_prod_util.h"
-#include "base/macros.h"
-#include "base/memory/ref_counted.h"
+#include "base/memory/raw_ptr.h"
+#include "base/memory/ref_counted_memory.h"
+#include "base/memory/scoped_refptr.h"
 #include "base/memory/weak_ptr.h"
-#include "base/timer/timer.h"
-#include "chrome/common/buildflags.h"
+#include "base/values.h"
+#include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
+#include "components/enterprise/buildflags/buildflags.h"
 #include "components/prefs/pref_service.h"
-#include "components/signin/public/identity_manager/identity_manager.h"
+#include "components/printing/common/print.mojom.h"
 #include "content/public/browser/web_ui_message_handler.h"
+#include "mojo/public/cpp/bindings/associated_remote.h"
 #include "printing/backend/print_backend.h"
 #include "printing/buildflags/buildflags.h"
+#include "printing/mojom/print.mojom.h"
+#include "printing/print_job_constants.h"
 
 namespace base {
-class DictionaryValue;
-class RefCountedMemory;
+class TimeTicks;
+}  // namespace base
+
+#if BUILDFLAG(IS_CHROMEOS)
+namespace crosapi::mojom {
+class LocalPrinter;
 }
 
+#endif
+
 namespace content {
 class WebContents;
 }
 
 namespace printing {
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+class ExtensionPrinterHandlerAdapterAsh;
+#endif
 class PdfPrinterHandler;
 class PrinterHandler;
 class PrintPreviewUI;
-
-// Must match print_preview.PrinterType in
-// chrome/browser/resources/print_preview/native_layer.js
-enum PrinterType {
-  kPrivetPrinter,
-  kExtensionPrinter,
-  kPdfPrinter,
-  kLocalPrinter,
-  kCloudPrinter
-};
+enum class UserActionBuckets;
 
 // The handler for Javascript messages related to the print preview dialog.
-class PrintPreviewHandler : public content::WebUIMessageHandler,
-                            public signin::IdentityManager::Observer {
+class PrintPreviewHandler : public content::WebUIMessageHandler {
  public:
   PrintPreviewHandler();
+  PrintPreviewHandler(const PrintPreviewHandler&) = delete;
+  PrintPreviewHandler& operator=(const PrintPreviewHandler&) = delete;
   ~PrintPreviewHandler() override;
 
   // WebUIMessageHandler implementation.
@@ -60,11 +69,6 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
   void OnJavascriptAllowed() override;
   void OnJavascriptDisallowed() override;
 
-  // IdentityManager::Observer implementation.
-  void OnAccountsInCookieUpdated(
-      const signin::AccountsInCookieJarInfo& accounts_in_cookie_jar_info,
-      const GoogleServiceAuthError& error) override;
-
   // Called when print preview failed. |request_id| identifies the request that
   // failed.
   void OnPrintPreviewFailed(int request_id);
@@ -86,7 +90,7 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
   // Send the print preset options from the document.
   void SendPrintPresetOptions(bool disable_scaling,
                               int copies,
-                              int duplex,
+                              mojom::DuplexMode duplex,
                               int request_id);
 
   // Send the print preview page count and fit to page scaling
@@ -95,8 +99,9 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
                           int request_id);
 
   // Send the default page layout
-  void SendPageLayoutReady(const base::DictionaryValue& layout,
-                           bool has_custom_page_size_style,
+  void SendPageLayoutReady(base::Value::Dict layout,
+                           bool all_pages_have_custom_size,
+                           bool all_pages_have_custom_orientation,
                            int request_id);
 
   // Notify the WebUI that the page preview is ready.
@@ -104,69 +109,62 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
                             int preview_uid,
                             int preview_request_id);
 
-  int regenerate_preview_request_count() const {
-    return regenerate_preview_request_count_;
-  }
-
   // Notifies PDF Printer Handler that |path| was selected. Used for tests.
-  void FileSelectedForTesting(const base::FilePath& path,
-                              int index,
-                              void* params);
+  void FileSelectedForTesting(const base::FilePath& path, int index);
 
   // Sets |pdf_file_saved_closure_| to |closure|.
   void SetPdfSavedClosureForTesting(base::OnceClosure closure);
 
-  // Fires the 'enable-manipulate-settings-for-test' WebUI event.
-  void SendEnableManipulateSettingsForTest();
-
-  // Fires the 'manipulate-settings-for-test' WebUI event with |settings|.
-  void SendManipulateSettingsForTest(const base::DictionaryValue& settings);
+  virtual PrinterHandler* GetPrinterHandler(mojom::PrinterType printer_type);
 
  protected:
-  // Protected so unit tests can override.
-  virtual PrinterHandler* GetPrinterHandler(PrinterType printer_type);
-  virtual bool IsCloudPrintEnabled();
-
   // Shuts down the initiator renderer. Called when a bad IPC message is
   // received.
   virtual void BadMessageReceived();
 
   // Gets the initiator for the print preview dialog.
-  virtual content::WebContents* GetInitiator() const;
+  // Virtual so tests can override.
+  virtual content::WebContents* GetInitiator();
 
-  // Register/unregister from notifications of changes done to the GAIA
-  // cookie. Protected so unit tests can override.
-  virtual void RegisterForGaiaCookieChanges();
-  virtual void UnregisterForGaiaCookieChanges();
+  // Initiates print after any content analysis checks have been passed
+  // successfully.
+  virtual void FinishHandleDoPrint(UserActionBuckets user_action,
+                                   base::Value::Dict settings,
+                                   scoped_refptr<base::RefCountedMemory> data,
+                                   const std::string& callback_id);
 
  private:
   friend class PrintPreviewPdfGeneratedBrowserTest;
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewPdfGeneratedBrowserTest,
                            MANUAL_DummyTest);
   friend class PrintPreviewHandlerTest;
-  FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, InitialSettingsSimple);
-  FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest,
-                           InitialSettingsEnableHeaderFooter);
-  FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest,
-                           InitialSettingsDisableHeaderFooter);
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, GetPrinters);
+  FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, GetNoDenyListPrinters);
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, GetPrinterCapabilities);
+  FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest,
+                           GetNoDenyListPrinterCapabilities);
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, Print);
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, GetPreview);
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerTest, SendPreviewUpdates);
   friend class PrintPreviewHandlerFailingTest;
   FRIEND_TEST_ALL_PREFIXES(PrintPreviewHandlerFailingTest,
                            GetPrinterCapabilities);
+  FRIEND_TEST_ALL_PREFIXES(ContentAnalysisPrintPreviewHandlerTest,
+                           LocalScanBeforePrinting);
+  content::WebContents* preview_web_contents();
 
-#if defined(OS_CHROMEOS)
-  class AccessTokenService;
-#endif
+  PrintPreviewUI* print_preview_ui();
 
-  content::WebContents* preview_web_contents() const;
+  const mojom::RequestPrintPreviewParams* GetRequestParams();
 
-  PrintPreviewUI* print_preview_ui() const;
+  PrefService* GetPrefs();
 
-  PrefService* GetPrefs() const;
+  // Checks policy preferences for a deny list of printer types and initializes
+  // the set that stores them.
+  void ReadPrinterTypeDenyListFromPrefs();
+
+  void OnPrinterTypeDenyListReady(
+      const std::vector<mojom::PrinterType>& deny_list_types);
 
   // Whether the the handler should be receiving messages from the renderer to
   // forward to the Print Preview JS in response to preview request with id
@@ -182,95 +180,60 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
 
   // Gets the list of printers. First element of |args| is the Javascript
   // callback, second element of |args| is the printer type to fetch.
-  void HandleGetPrinters(const base::ListValue* args);
-
-  // Grants an extension access to a provisional printer.  First element of
-  // |args| is the provisional printer ID.
-  void HandleGrantExtensionPrinterAccess(const base::ListValue* args);
+  void HandleGetPrinters(const base::Value::List& args);
 
   // Asks the initiator renderer to generate a preview.  First element of |args|
   // is a job settings JSON string.
-  void HandleGetPreview(const base::ListValue* args);
+  void HandleGetPreview(const base::Value::List& args);
 
   // Gets the job settings from Web UI and initiate printing. First element of
   // |args| is a job settings JSON string.
-  void HandlePrint(const base::ListValue* args);
+  void HandleDoPrint(const base::Value::List& args);
 
   // Handles the request to hide the preview dialog for printing.
   // |args| is unused.
-  void HandleHidePreview(const base::ListValue* args);
+  void HandleHidePreview(const base::Value::List& args);
 
   // Handles the request to cancel the pending print request. |args| is unused.
-  void HandleCancelPendingPrintRequest(const base::ListValue* args);
+  void HandleCancelPendingPrintRequest(const base::Value::List& args);
 
   // Handles a request to store data that the web ui wishes to persist.
   // First element of |args| is the data to persist.
-  void HandleSaveAppState(const base::ListValue* args);
+  void HandleSaveAppState(const base::Value::List& args);
 
   // Gets the printer capabilities. Fist element of |args| is the Javascript
   // callback, second element is the printer ID of the printer whose
   // capabilities are requested, and the third element is the type of the
   // printer whose capabilities are requested.
-  void HandleGetPrinterCapabilities(const base::ListValue* args);
-
-  // Performs printer setup. First element of |args| is the printer name.
-  void HandlePrinterSetup(const base::ListValue* args);
+  void HandleGetPrinterCapabilities(const base::Value::List& args);
 
 #if BUILDFLAG(ENABLE_BASIC_PRINT_DIALOG)
   // Asks the initiator renderer to show the native print system dialog. |args|
   // is unused.
-  void HandleShowSystemDialog(const base::ListValue* args);
-#endif
-
-  // Opens a new tab to allow the user to sign into cloud print. |args| holds
-  // a boolean indicating whether the user is adding an account.
-  void HandleSignin(const base::ListValue* args);
-
-  // Called when the tab opened by HandleSignIn() is closed.
-  void OnSignInTabClosed();
-
-#if defined(OS_CHROMEOS)
-  // Generates new token and sends back to UI.
-  void HandleGetAccessToken(const base::ListValue* args);
+  void HandleShowSystemDialog(const base::Value::List& args);
 #endif
 
   // Gathers UMA stats when the print preview dialog is about to close.
   // |args| is unused.
-  void HandleClosePreviewDialog(const base::ListValue* args);
+  void HandleClosePreviewDialog(const base::Value::List& args);
 
   // Asks the browser for several settings that are needed before the first
   // preview is displayed.
-  void HandleGetInitialSettings(const base::ListValue* args);
+  void HandleGetInitialSettings(const base::Value::List& args);
 
-#if defined(OS_CHROMEOS)
-  // Opens printer settings in the Chrome OS Settings App.
-  void HandleOpenPrinterSettings(const base::ListValue* args);
-#endif
+  // Opens printer settings in the Chrome OS Settings App or OS's printer manger
+  // dialog. |args| is unused.
+  void HandleManagePrinters(const base::Value::List& args);
 
   void SendInitialSettings(const std::string& callback_id,
+                           base::Value::Dict policies,
                            const std::string& default_printer);
 
-#if defined(OS_CHROMEOS)
-  // Send OAuth2 access token.
-  void SendAccessToken(const std::string& callback_id,
-                       const std::string& access_token);
-#endif
-
   // Sends the printer capabilities to the Web UI. |settings_info| contains
   // printer capabilities information. If |settings_info| is empty, sends
   // error notification to the Web UI instead.
   void SendPrinterCapabilities(const std::string& callback_id,
-                               base::Value settings_info);
-
-  // Send the result of performing printer setup. |settings_info| contains
-  // printer capabilities.
-  void SendPrinterSetup(const std::string& callback_id,
-                        const std::string& printer_name,
-                        base::Value settings_info);
-
-  // Send the PDF data to the cloud to print.
-  void SendCloudPrintJob(const std::string& callback_id,
-                         const base::RefCountedMemory* data);
+                               base::Value::Dict settings_info);
 
   // Closes the preview dialog.
   void ClosePreviewDialog();
@@ -279,10 +242,10 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
   void ClearInitiatorDetails();
 
   // Populates |settings| according to the current locale.
-  void GetLocaleInformation(base::Value* settings);
+  void GetLocaleInformation(base::Value::Dict* settings);
 
   // Populates |settings| with the list of logged in accounts.
-  void GetUserAccountList(base::Value* settings);
+  void GetUserAccountList(base::Value::Dict* settings);
 
   PdfPrinterHandler* GetPdfPrinterHandler();
 
@@ -290,67 +253,49 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
   // |printer_type|: The type of printers that were added.
   // |printers|: A non-empty list containing information about the printer or
   //     printers that have been added.
-  void OnAddedPrinters(PrinterType printer_type,
-                       const base::ListValue& printers);
+  void OnAddedPrinters(mojom::PrinterType printer_type,
+                       base::Value::List printers);
 
   // Called when printer search is done for some destination type.
   // |callback_id|: The javascript callback to call.
-  void OnGetPrintersDone(const std::string& callback_id);
+  void OnGetPrintersDone(const std::string& callback_id,
+                         mojom::PrinterType printer_type,
+                         const base::TimeTicks& start_time);
 
-  // Called when an extension reports information requested for a provisional
-  // printer.
-  // |callback_id|: The javascript callback to resolve or reject.
-  // |printer_info|: The data reported by the extension.
-  void OnGotExtensionPrinterInfo(const std::string& callback_id,
-                                 const base::DictionaryValue& printer_info);
-
-  // Called when an extension or privet print job is completed.
+  // Called when an extension print job is completed.
   // |callback_id|: The javascript callback to run.
   // |error|: The returned print job error. Useful for reporting a specific
   //     error. None type implies no error.
   void OnPrintResult(const std::string& callback_id,
                      const base::Value& error);
 
-  // A count of how many requests received to regenerate preview data.
-  // Initialized to 0 then incremented and emitted to a histogram.
-  int regenerate_preview_request_count_;
+#if BUILDFLAG(ENTERPRISE_CONTENT_ANALYSIS)
+  // Called when enterprise policy returns a verdict.
+  // Calls FinishHandleDoPrint() if it's allowed or calls OnPrintResult() to
+  // report print not allowed.
+  void OnVerdictByEnterprisePolicy(UserActionBuckets user_action,
+                                   base::Value::Dict settings,
+                                   scoped_refptr<base::RefCountedMemory> data,
+                                   const std::string& callback_id,
+                                   bool allowed);
 
-  // A count of how many requests received to show manage printers dialog.
-  int manage_printers_dialog_request_count_;
+  // Wrapper for OnHidePreviewDialog() from PrintPreviewUI.
+  void OnHidePreviewDialog();
+#endif  // BUILDFLAG(ENTERPRISE_CONTENT_ANALYSIS)
 
   // Whether we have already logged a failed print preview.
-  bool reported_failed_preview_;
+  bool reported_failed_preview_ = false;
 
   // Whether we have already logged the number of printers this session.
-  bool has_logged_printers_count_;
-
-  // Whether Google Cloud Print is enabled for the active profile.
-  bool cloud_print_enabled_ = false;
+  bool has_logged_printers_count_ = false;
 
   // The settings used for the most recent preview request.
-  base::Value last_preview_settings_;
-
-#if defined(OS_CHROMEOS)
-  // Holds token service to get OAuth2 access tokens.
-  std::unique_ptr<AccessTokenService> token_service_;
-#endif
-
-  // Pointer to the identity manager service so that print preview can listen
-  // for GAIA cookie changes.
-  signin::IdentityManager* identity_manager_;
-
-  // Handles requests for cloud printers. Created lazily by calling
-  // GetPrinterHandler().
-  std::unique_ptr<PrinterHandler> cloud_printer_handler_;
+  std::optional<base::Value::Dict> last_preview_settings_;
 
   // Handles requests for extension printers. Created lazily by calling
   // GetPrinterHandler().
   std::unique_ptr<PrinterHandler> extension_printer_handler_;
 
-  // Handles requests for privet printers. Created lazily by calling
-  // GetPrinterHandler().
-  std::unique_ptr<PrinterHandler> privet_printer_handler_;
-
   // Handles requests for printing to PDF. Created lazily by calling
   // GetPrinterHandler().
   std::unique_ptr<PrinterHandler> pdf_printer_handler_;
@@ -360,14 +305,39 @@ class PrintPreviewHandler : public content::WebUIMessageHandler,
   std::unique_ptr<PrinterHandler> local_printer_handler_;
 
   // Maps preview request ids to callbacks.
-  std::map<int, std::string> preview_callbacks_;
+  base::flat_map<int, std::string> preview_callbacks_;
 
   // Set of preview request ids for failed previews.
-  std::set<int> preview_failures_;
+  base::flat_set<int> preview_failures_;
+
+  // Set of printer types on the deny list.
+  base::flat_set<mojom::PrinterType> printer_type_deny_list_;
+
+  // Used to transmit mojo interface method calls to the associated receiver.
+  mojo::AssociatedRemote<mojom::PrintRenderFrame> print_render_frame_;
+
+#if BUILDFLAG(IS_CHROMEOS)
+  // Used to transmit mojo interface method calls to ash chrome.
+  // Null if the interface is unavailable.
+  // Note that this is not propagated to LocalPrinterHandlerLacros.
+  // The pointer is constant - if ash crashes and the mojo connection is lost,
+  // lacros will automatically be restarted.
+  raw_ptr<crosapi::mojom::LocalPrinter, DanglingUntriaged> local_printer_ =
+      nullptr;
+#endif
+
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  // Used when Lacros is enabled.
+  std::unique_ptr<ExtensionPrinterHandlerAdapterAsh>
+      extension_printer_handler_adapter_;
+#endif
+
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+  // Version number of the LocalPrinter mojo service.
+  int local_printer_version_ = 0;
+#endif
 
   base::WeakPtrFactory<PrintPreviewHandler> weak_factory_{this};
-
-  DISALLOW_COPY_AND_ASSIGN(PrintPreviewHandler);
 };
 
 }  // namespace printing
